//-----------------------------------------------------------------------
// <copyright file="Startup.cs" company="TechCello">
// Copyright (c) TechCello. All rights reserved.
// <author> TechCello </author>
// <date>8/24/2014 3:40:54 PM</date>
// </copyright>
//-----------------------------------------------------------------------

using CelloSaaS.AuthorizationServer.App_Start;
using CelloSaaS.AuthServer.Core;
using CelloSaaS.AuthServer.Core.Ancilliary;
using CelloSaaS.AuthServer.Core.Contracts.Claims;
using CelloSaaS.AuthServer.Core.Contracts.Clients;
using CelloSaaS.AuthServer.Core.Middlewares;
using CelloSaaS.Library;
using CelloSaaS.Owin.Security.GenericOAuth2Authentication;
using CelloSaaS.Owin.Security.GenericOAuth2Authentication.Context;
using CelloSaaS.Owin.Security.GenericOAuth2Authentication.Provider;
using CelloSaaS.Owin.Security.GenericOAuthMiddleware.Profile;
using Microsoft.Owin;
using Microsoft.Owin.Security;
using Microsoft.Owin.Security.Cookies;
using Microsoft.Owin.Security.Facebook;
using Microsoft.Owin.Security.Google;
using Microsoft.Owin.Security.Twitter;
using Owin;
using System.Collections.Generic;
using System.Security.Claims;
using System.Web.Http;

namespace CelloSaaS.AuthorizationServer
{
    public class Startup
    {
        public void Configuration(IAppBuilder app)
        {
            AuthenticationMiddlewares(app);

            app.UseWebApi(WebApiConfig.Configure(new HttpConfiguration()));
        }

        private static void AuthenticationMiddlewares(IAppBuilder app)
        {
            var cookieAuthOptions = new CookieAuthenticationOptions
            {
                AuthenticationType = AuthServerConstants.PrimaryAuthenticationType,
            };

            app.UseCookieAuthentication(cookieAuthOptions);

            app.UseExternalSignInCookie(AuthServerConstants.ExternalAuthenticationType);

            ConfigureExternalIdentityProviders(app, AuthServerConstants.ExternalAuthenticationType);
        }

        public static void ConfigureExternalIdentityProviders(IAppBuilder app, string signInAsType)
        {
            var claimTransformer = ServiceLocator.Resolve<IClaimsTransformer>();

            #region LinkedIn Auth

            var linkedInAuthOptions = new GenericOAuth2AuthenticationOptions("LinkedIn")
            {
                Caption = "LinkedIn",
                SignInAsAuthenticationType = signInAsType,
                AccessType = "LinkedIn",
                Scope = new List<string> { "all" },
                AuthenticationMode = Microsoft.Owin.Security.AuthenticationMode.Passive,
                SkipScopeInRequests = true,
                Mode = OperationalMode.OAuth2,
                ClientId = AppSettingHelpers.GetValue<string>(AppSettingConstants.LinkedInAppId),
                ClientSecret = AppSettingHelpers.GetValue<string>(AppSettingConstants.LinkedInAppSecret),
                //IncludeStateInTokenRequests = true,
                IncludeClientIdInProfileRequests = false,
                AuthorizationEndpointUri = "https://www.linkedin.com/uas/oauth2/authorization",
                TokenEndpointUri = "https://www.linkedin.com/uas/oauth2/accessToken",
                UserInfoEndpointUri = "https://api.linkedin.com/v1/people/~:(id,first-name,last-name,formatted-name,email-address,location,public-profile-url)?format=json"

            };
            linkedInAuthOptions.Provider = new GenericOAuth2AuthenticationProvider
            {
                InitRequestFromAuthenticationProperties = EndpointUriTypes => true,
                EnforceResponseMimeType = endpointUriTypes =>
                {
                    switch (endpointUriTypes) {
                        default:
                        case CelloSaaS.Owin.Security.GenericOAuthMiddleware.Provider.EndpointUriTypes.AuthorizeEndpointUri:
                        case CelloSaaS.Owin.Security.GenericOAuthMiddleware.Provider.EndpointUriTypes.TokenEndpointUri:
                            return false;
                        case CelloSaaS.Owin.Security.GenericOAuthMiddleware.Provider.EndpointUriTypes.UserInfoEndpointUri:
                            return true;
                    }
                },
                OnMapContextToClaims = context =>
                {
                    return System.Threading.Tasks.Task.FromResult<List<Claim>>(claimTransformer.GetAdditionalClaimsFromContext(context));
                },
                OnAuthenticated = async context =>
                {
                    claimTransformer.TransformClaims<GenericOAuth2AuthenticatedContext, GenericOAuth2AuthenticationOptions>(context, linkedInAuthOptions);
                },
                OnReturnEndpoint = async context =>
                {
                },
                TokenRequestContent = (body) =>
                {
                    return new System.Net.Http.FormUrlEncodedContent(body);
                },
                //AppendAuthorizeRequestParamters = (text, queryParameters) =>
                //{
                //    queryParameters.Add("resource", "https://graph.windows.net");
                //    return queryParameters;
                //},
                // Specific for Azure Active Directory
                ParseUserProfile = (text) =>
                {
                    var linkedInProfileParser = ServiceLocator.Resolve<IProfileParser>("LinkedInProfileParser");
                    return linkedInProfileParser.ParseFullProfile(text);
                }
            };

            app.UseGenericOAuth2Authentication(linkedInAuthOptions);

            #endregion

            #region WAAD Auth

            var waadAuthOptions = new GenericOAuth2AuthenticationOptions("AzureActiveDirectory")
            {
                Caption = "AzureActiveDirectory",
                SignInAsAuthenticationType = signInAsType,
                AccessType = "AzureActiveDirectory",
                Scope = new List<string> { "all" },
                AuthenticationMode = Microsoft.Owin.Security.AuthenticationMode.Passive,
                SkipScopeInRequests = true,
                Mode = OperationalMode.OAuth2,
                ClientId = "9242e79d-3044-4366-a4b6-56b3ae952bd4",
                ClientSecret = "k2wpTug0hFz/lFDTJ3z7xkYu+OYvHHX25rb+e4HvU0o=",
                IncludeStateInTokenRequests = true,
                IncludeClientIdInProfileRequests = false
            };
            waadAuthOptions.Provider = new GenericOAuth2AuthenticationProvider
            {
                InitRequestFromAuthenticationProperties = EndpointUriTypes => true,
                OnDemandEndpoints = async (clientid, EndpointUriTypes) =>
                {
                    var endpointResolver = ServiceLocator.Resolve<IClientEndpointResolver>();
                    return await endpointResolver.ResolveEndpointUri(EndpointUriTypes, clientid);
                },
                OnMapContextToClaims = context =>
                {
                    return System.Threading.Tasks.Task.FromResult<List<Claim>>(claimTransformer.GetAdditionalClaimsFromContext(context));
                },
                OnAuthenticated = async context =>
                {
                    claimTransformer.TransformClaims<GenericOAuth2AuthenticatedContext, GenericOAuth2AuthenticationOptions>(context, waadAuthOptions);
                },
                OnReturnEndpoint = async context =>
                {
                },
                TokenRequestContent = (body) =>
                {
                    return new System.Net.Http.FormUrlEncodedContent(body);
                },
                AppendAuthorizeRequestParamters = (text, queryParameters) =>
                {
                    queryParameters.Add("resource", "https://graph.windows.net");
                    return queryParameters;
                },
                // Specific for Azure Active Directory
                ParseUserProfile = (text) =>
                {
                    var activeDirectoryProfileParser = ServiceLocator.Resolve<IProfileParser>("AzureActiveDirectoryProfileParser");
                    return activeDirectoryProfileParser.ParseFullProfile(text);
                }
            };

            app.UseGenericOAuth2Authentication(waadAuthOptions);

            #endregion

            #region ADFS3Native Auth

            var adfs3NativeOptions = new GenericOAuth2AuthenticationOptions("ADFSNative")
            {
                Caption = "ADFSNative",
                SignInAsAuthenticationType = signInAsType,
                AccessType = "ADFSNative",
                Scope = new List<string> { "all" },
                AuthenticationMode = Microsoft.Owin.Security.AuthenticationMode.Passive,
                SkipScopeInRequests = true,
                Mode = OperationalMode.OpenIdConnect1
            };
            adfs3NativeOptions.Provider = new GenericOAuth2AuthenticationProvider
            {
                InitRequestFromAuthenticationProperties = EndpointUriTypes => true,
                OnDemandEndpoints = async (clientid, EndpointUriTypes) =>
                {
                    var endpointResolver = ServiceLocator.Resolve<IClientEndpointResolver>();
                    return await endpointResolver.ResolveEndpointUri(EndpointUriTypes, clientid, OperationalMode.OpenIdConnect1);
                },
                OnMapContextToClaims = context =>
                {
                    return System.Threading.Tasks.Task.FromResult<List<Claim>>(claimTransformer.GetAdditionalClaimsFromContext(context));
                },
                OnAuthenticated = async context =>
                {
                    claimTransformer.TransformClaims<GenericOAuth2AuthenticatedContext, GenericOAuth2AuthenticationOptions>(context, adfs3NativeOptions);
                },
                OnReturnEndpoint = async context =>
                {
                },
                TokenRequestContent = (body) =>
                {
                    return new System.Net.Http.FormUrlEncodedContent(body);
                },
                AppendAuthorizeRequestParamters = (text, queryParameters) =>
                {
                    queryParameters.Add("resource", System.Configuration.ConfigurationManager.AppSettings["AuthIssuer"]);
                    return queryParameters;
                }
            };

            app.UseGenericOAuth2Authentication(adfs3NativeOptions);

            #endregion

            #region LDAP Auth
            //{"AuthorizationEndPointUri":"https://identity.cello.com:44330/Authorize"
            //,"ProfileEndPointUri":"https://identity.cello.com:44320/Authentication/Login"}
            var ldapAuthOptions = new GenericOAuth2AuthenticationOptions("LDAP")
            {
                Caption = "LDAP",
                SignInAsAuthenticationType = signInAsType,
                AccessType = "LDAP",
                Scope = new List<string> { "all" },
                AuthenticationMode = Microsoft.Owin.Security.AuthenticationMode.Passive
            };
            ldapAuthOptions.Provider = new GenericOAuth2AuthenticationProvider
            {
                InitRequestFromAuthenticationProperties = EndpointUriTypes => true,
                OnDemandEndpoints = async (clientid, EndpointUriTypes) =>
                {
                    var endpointResolver = ServiceLocator.Resolve<IClientEndpointResolver>();
                    return await endpointResolver.ResolveEndpointUri(EndpointUriTypes, clientid);
                },
                OnMapContextToClaims = context =>
                {
                    return System.Threading.Tasks.Task.FromResult<List<Claim>>(claimTransformer.GetAdditionalClaimsFromContext(context));
                },
                OnAuthenticated = async context =>
                {
                    claimTransformer.TransformClaims<GenericOAuth2AuthenticatedContext, GenericOAuth2AuthenticationOptions>(context, ldapAuthOptions);
                },
                OnReturnEndpoint = async context =>
                {
                }
            };

            app.UseGenericOAuth2Authentication(ldapAuthOptions);

            #endregion

            #region Google Auth
            var google = new GoogleOAuth2AuthenticationOptions
               {
                   ClientId = AppSettingHelpers.GetValue<string>(AppSettingConstants.GoogleAppId),
                   ClientSecret = AppSettingHelpers.GetValue<string>(AppSettingConstants.GoogleAppSecret),
                   SignInAsAuthenticationType = AuthServerConstants.ExternalAuthenticationType,
                   AuthenticationMode = AuthenticationMode.Passive
               };
            google.Provider = new GoogleOAuth2AuthenticationProvider
            {
                OnAuthenticated = async ctx =>
                {
                    claimTransformer.TransformClaims<GoogleOAuth2AuthenticatedContext, GoogleOAuth2AuthenticationOptions>(ctx, google);
                }
            };
            app.UseGoogleAuthentication(google);
            #endregion

            #region Facebook Auth
            var fbOptions = new FacebookAuthenticationOptions
            {
                AppId = AppSettingHelpers.GetValue<string>(AppSettingConstants.FacebookAppId),
                AppSecret = AppSettingHelpers.GetValue<string>(AppSettingConstants.FacebookAppSecret),
                SignInAsAuthenticationType = AuthServerConstants.ExternalAuthenticationType,
                AuthenticationMode = AuthenticationMode.Passive
            };

            fbOptions.Scope.Add("email");
            fbOptions.Provider = new FacebookAuthenticationProvider
            {
                OnAuthenticated = async ctx =>
                {
                    claimTransformer.TransformClaims<FacebookAuthenticatedContext, FacebookAuthenticationOptions>(ctx, fbOptions);
                }
            };

            app.UseFacebookAuthentication(fbOptions);
            #endregion

            #region Twitter Auth
            var twitterOptions = new TwitterAuthenticationOptions
               {
                   ConsumerKey = AppSettingHelpers.GetValue<string>(AppSettingConstants.TwitterAppId),
                   ConsumerSecret = AppSettingHelpers.GetValue<string>(AppSettingConstants.TwitterAppSecret),
                   SignInAsAuthenticationType = AuthServerConstants.ExternalAuthenticationType,
                   AuthenticationMode = AuthenticationMode.Passive
               };
            twitterOptions.Provider = new TwitterAuthenticationProvider
            {
                OnAuthenticated = async ctx =>
                {
                    claimTransformer.TransformClaims<TwitterAuthenticatedContext, TwitterAuthenticationOptions>(ctx, twitterOptions);
                }
            };

            app.UseTwitterAuthentication(twitterOptions);
            #endregion
        }
    }
}