//-----------------------------------------------------------------------
// <copyright file="WebApiConfig.cs" company="TechCello">
// Copyright (c) TechCello. All rights reserved.
// <author> TechCello </author>
// <date>8/24/2014 3:40:54 PM</date>
// </copyright>
//-----------------------------------------------------------------------

using System.Collections.Generic;
using System.Web.Http;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;
using CelloSaaS.AuthServer.Core.Extensions;
using CelloSaaS.AuthServer.Core.Performance;
using System.Threading.Tasks;
using CelloSaaS.Configuration;

namespace CelloSaaS.AuthorizationServer.App_Start
{
    public static class WebApiConfig
    {
        public static void Register(HttpConfiguration config)
        {
            GlobalConfiguration.Configuration.Services.Replace(typeof(IHttpActionInvoker), new CelloApiControllerActionInvoker());
            CelloConfiguration.Configure();
            ConfigureWebApi(config);
            config.MessageHandlers.Add(new CelloSaaS.AuthServer.Core.Security.MessageHandlers.ThrottlingHandler());
            config.EnsureInitialized();
        }

        private static void ConfigureWebApi(HttpConfiguration config)
        {
            // Web API configuration and services
            // TODO :Not used as of now due to the exceptions raised for built in service mappings
            config.EnableCors();

            // Web API routes
            config.MapHttpAttributeRoutes();

            config.Routes.MapHttpRoute(
                name: "DefaultApi",
                routeTemplate: "api/{controller}/{id}",
                defaults: new { id = RouteParameter.Optional }
            );
            //Register the default filters here
            config.Filters.AddRange(new List<IFilter> { new CelloApiActionTimers(), new ExceptionHandlingAttribute() });
            config.MessageHandlers.Add(new CelloSaaS.AuthServer.Core.Security.MessageHandlers.ThrottlingHandler());
        }

        public static HttpConfiguration Configure(HttpConfiguration config)
        {
            GlobalConfiguration.Configuration.Services.Replace(typeof(IHttpActionInvoker), new CelloApiControllerActionInvoker());
            ConfigureWebApi(config);
            CelloConfiguration.Configure();
            return config;
        }
    }
}
