﻿using CelloSaaS.AuthorizationServer.Models;
using CelloSaaS.AuthServer.Core;
using CelloSaaS.AuthServer.Core.Ancilliary;
using CelloSaaS.AuthServer.Core.Results.Errors;
using CelloSaaS.AuthServer.Core.Services.CacheServices;
using CelloSaaS.AuthServer.Core.Services.Clients;
using System;
using System.Web.Http;

namespace CelloSaaS.AuthorizationServer.Controllers
{
    [RoutePrefix(Routes.CacheRoutePrefix)]
    public class CacheController : ApiController
    {
        [HttpGet]
        public IHttpActionResult Clear()
        {
            CelloSaaS.AuthServer.Core.Ancilliary.ResourceManagement.AuthServerResourceManager.Flush();
            FlushObjectCache();
            FlushResourcesCache();
            FlushAppSettings();
            ClearUserCache();
            return Ok<string>(Localization.CacheResource.msg_CacheClearSuccess);
        }

        [Route(Routes.ClearResourceCacheRoute)]
        [HttpGet]
        public IHttpActionResult ClearResourceCache()
        {
            try
            {
                CelloSaaS.AuthServer.Core.Ancilliary.ResourceManagement.AuthServerResourceManager.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult(Localization.CacheResource.ex_CacheClearFail);
            }
            return Ok<string>(Localization.CacheResource.msg_RsrcCacheClearSuccess);
        }

        [Route(Routes.ClearUserCacheRoute)]
        [HttpGet]
        public IHttpActionResult ClearUserCache()
        {
            try
            {
                CelloUserCacheService.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult(Localization.CacheResource.ex_CacheClearFail);
            }
            return Ok<string>(Localization.CacheResource.msg_UserCacheClearSuccess);
        }

        [Route(Routes.ClearObjectCacheRoute)]
        [HttpGet]
        public IHttpActionResult FlushObjectCache()
        {
            try
            {
                AuthenticationProvidersCacheService.Flush();
                AuthenticationTypesCacheService.Flush();
                ClientCacheService.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult(Localization.CacheResource.ex_CacheClearFail);
            }
            return Ok<string>(Localization.CacheResource.msg_ObjCacheClearSuccess);
        }

        [Route(Routes.ClearResourceCacheRoute)]
        [HttpGet]
        public IHttpActionResult FlushResourcesCache()
        {
            try
            {
                StaticResourceHelpers.Flush();
                StaticResourceHelpers.Init();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult(Localization.CacheResource.ex_CacheClearFail);
            }
            return Ok<string>(Localization.CacheResource.msg_RsrcCacheClearSuccess);
        }

        [Route(Routes.ClearApplicationSettings)]
        [HttpGet]
        public IHttpActionResult FlushAppSettings()
        {
            try
            {
                AppSettingHelpers.Flush();
            }
            catch (Exception exception)
            {
                System.Threading.Tasks.Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingConstants.AuthServerExceptionPolicy));
                return new ErrorResult(Localization.CacheResource.ex_CacheClearFail);
            }
            return Ok<string>(Localization.CacheResource.msg_AppCacheClearSuccess);
        }
    }

    
}
